/*
 * DataTables4j, a JSP taglib to display table with jQuery and DataTables
 * Copyright (c) 2012, DataTables4j <datatables4j@gmail.com>
 *
 * This Program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * The Program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package com.github.datatables4j.plugins.ajax;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.datatables4j.core.api.constants.DTConstants;
import com.github.datatables4j.core.api.datasource.AjaxSource;
import com.github.datatables4j.core.api.exception.DataNotFoundException;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;

/**
 * Jersey implementation of DataProvider for AJAX table.
 * 
 * @author Thibault Duchateau
 */
public class JerseyDataProvider implements AjaxSource {

	// Logger
	private static Logger logger = LoggerFactory.getLogger(JerseyDataProvider.class);
		
	/**
	 * TODO
	 */
	public Object getData(String wsUrl) throws DataNotFoundException {

		logger.debug("Retrieving data with JerseyDataProvider");
		
		// Jersey client initialisation
		Client client = Client.create();

		// WebResource initialisation
		WebResource webResource = client.resource(wsUrl);
		
		logger.debug("Web service call ...");
		
		// Performing GET request
		ClientResponse response = webResource.accept("application/json").get(
				ClientResponse.class);
		if (response.getStatus() != 200) {
			throw new DataNotFoundException("No data found : " + String.valueOf(response.getStatus()));
		}
		
		String output = response.getEntity(String.class);
		System.out.println("output = "+ output);
		// Jackson object mapper
		ObjectMapper mapper = new ObjectMapper();

		Map<String, Object> map = new HashMap<String, Object>();
		JsonNode df;

		try {
			logger.debug("Converting response to JSON ...");
			df = mapper.readValue(output, JsonNode.class);
			
			logger.debug("Convertion OK");
			map.put(DTConstants.DT_DS_DATA, df);
			
		} catch (JsonParseException e) {
			throw new DataNotFoundException(e);
		} catch (JsonMappingException e) {
			throw new DataNotFoundException(e);
		} catch (IOException e) {
			throw new DataNotFoundException(e);
		}

		return df;
	}
}